
#    Copyright (C) 2011  Stefano Palazzo <stefano.palazzo@gmail.com>

#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.

#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.

#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

from google_api import google_search

from htmlentitydefs import name2codepoint

NUMS = "0123456789"


# TODO
#  - Use name2codepoint to clean up the title; but only after making
#    sure it does the right things.


def clean_title(title):
    # This method is very sensitive to changes in the google search API.
    # It should be the first place to look if results "look odd".
    title = title.replace(u"- Ask Ubuntu", "")
    title = title.replace(u"- Ask ...", "")
    title = title.replace(u"- Stack Exchange", "")
    title = title.replace(u"- Stack ...", "")
    title = title.replace(u"&#39;", "'")
    title = title.replace(u"&amp;", "&")
    title = title.replace(u"&lt;", "<")
    title = title.replace(u"&gt;", ">")
    title = title.replace(u"&quot;", "\"")
    return title.strip()


# It should be possible to write a good test for these next four
# functions. Maybe check apps.se for a compilation of weird urls.
# A bad test will hurt more than it helps; they are simple enough.


def is_question_page(url):
    # Example: http://askubuntu.com/questions/131321/some-title-blah
    return ("askubuntu.com/questions/" in url and
        all(i in NUMS for i in url.split("questions/")[1].split("/")[0]) and
        not "meta.askubuntu.com/" in url)


def is_tags_page(url):
    # Example: http://askubuntu.com/questions/tagged/unity
    return ("askubuntu.com/tags/" in url and
        not "meta.askubuntu.com/" in url)


def is_users_page(url):
    # Example: http://askubuntu.com/users/131321/stefano-palazzo
    return ("askubuntu.com/users/" in url and
        all(i in NUMS for i in url.split("users/")[1].split("/")[0]) and
        not "meta.askubuntu.com/" in url)


def is_badges_page(url):
    # Example: http://askubuntu.com/badges/48/altruist
    return ("askubuntu.com/badges/" in url and
        all(i in NUMS for i in url.split("badges/")[1].split("/")[0]) and
        not "meta.askubuntu.com/" in url)


def get_questions(search):
    '''
    Returns Question on AskUbuntu matching the 'search' query.
    The yields 2-tuples of (url, title). It's a clean title that
    can be displayed to the end-user.

    '''
    print "starting search on google"
    data = google_search("%s site:askubuntu.com/questions/" % search)
    print "finished searching"

    for i in data:  # this is the empty list if there aren't any results
        url, title = i['url'], i['titleNoFormatting']
        if is_question_page(url):
            title = clean_title(title)
            if ' - ' in title:
                title = ' - '.join(title.split(u" - ")[1:])  # remove tag
            yield url, title


def get_tags(search):
    '''
    Returns Tags on AskUbuntu matching the 'search' query.
    The yields 2-tuples of (url, title). It's a clean title that
    can be displayed to the end-user.

    '''
    print "starting search for tags on google"
    data = google_search("%s site:askubuntu.com/tags" % search)
    print "finished searching"
    already_seen = []
    for i in data:  # this is the empty list if there aren't any results
        url, title = i['url'], i['titleNoFormatting']
        title = clean_title(title)
        if is_tags_page(url) and "'" in title:
            title = title.split("'")[1]
            url = '/'.join(url.split('/')[:5])
            if title not in already_seen:
                yield url, title
                already_seen.append(title)


def get_users(search):
    '''
    Returns Users on AskUbuntu matching the 'search' query.
    The yields 2-tuples of (url, title). It's a clean title that
    can be displayed to the end-user.

    '''
    print "starting search for users on google"
    data = google_search("%s site:askubuntu.com/users" % search)
    print "finished searching"
    already_seen = []
    for i in data:  # this is the empty list if there aren't any results
        url, title = i['url'], i['titleNoFormatting']
        title = clean_title(title)
        if is_users_page(url) and "User " in title:
            title = title.split("User ")[1]
            if title not in already_seen:
                yield url, title
                already_seen.append(title)


def get_badges(search):
    '''
    Returns Badges on AskUbuntu matching the 'search' query.
    The yields 2-tuples of (url, title). It's a clean title that
    can be displayed to the end-user.

    '''
    print "starting search for badges on google"
    data = google_search("%s site:askubuntu.com/badges" % search)
    print "finished searching"
    already_seen = []
    for i in data:  # this is the empty list if there aren't any results
        url, title = i['url'], i['titleNoFormatting']
        title = clean_title(title)
        if is_badges_page(url) and " Badge" in title:
            title = title.split(" Badge")[0]
            if title not in already_seen:
                url = url.split("%3F")[0]
                yield url, title
                already_seen.append(title)
                print url
